# test_flagdir.py
'''
Unit tests for jobStatus class.

Basic rules for jobstatus.jobStatus to follow:
1) It should be called as a context manager
2) If a status already exists, it exits without touching it



'''
import unittest
import tempfile
import shutil
import os

from ews.coordinator.utils import jobstatus


class TestJobstatus(unittest.TestCase):

    """Functions to test file and directory manipulation."""

    # TODO: replace this SetUp and tearDown approach with a context manager, for more robust workspace deletion
    def setUp(self):
        # TODO: Is it worth placing this temporary workspace in the project directory?
        self.jobdir = tempfile.mkdtemp()


    def tearDown(self):
        shutil.rmtree(self.jobdir)


    def test_basic(self):
        '''
        Basic test of unittest functionality.
        '''
        self.assertEqual(True, True)


    def test_context_call_assigns_status(self):
        with jobstatus.Jobstatus(self.jobdir) as js:
            self.assertTrue(hasattr(js, 'status'))


    def test_direct_call_does_not_assign_status(self):
        # because it should be used as a context manager

        self.assertFalse(hasattr(jobstatus.Jobstatus(self.jobdir), 'status'))


    def test_init_fails_when_path_not_exists(self):
        self.assertRaises(FileNotFoundError, jobstatus.Jobstatus, self.jobdir + 'fake')


    def test_get_from_file_path_fails_if_bad_file_status(self):
        # make a status file in job directory
        status_path = self.jobdir + '/STATUS_UNKNOWN'
        with open(status_path, 'w'): pass

        self.assertRaises(KeyError, jobstatus.Jobstatus.get_from_file_path, jobstatus.Jobstatus, status_path)


    def test_set_can_create_status_file(self):
        js = jobstatus.Jobstatus(self.jobdir)

        status_path_expected = self.jobdir + '/STATUS_SUCCESS'
        self.assertFalse(os.path.isfile(status_path_expected))

        js.set('SUCCESS')

        self.assertTrue(os.path.isfile(status_path_expected))


    def test_get_gets_status_if_file_is_present(self):
        # make a status file in job directory
        statusPath = self.jobdir + '/STATUS_SUCCESS'
        with open(statusPath, 'w'): pass

        js = jobstatus.Jobstatus(self.jobdir)

        self.assertIsInstance(js.get(), str)


    def test_cleanup_can_remove_status(self):
        # make a status file in job directory
        status_path_start = self.jobdir + '/STATUS_INPROGRESS'
        with open(status_path_start, 'w'): pass
        self.assertTrue(os.path.isfile(status_path_start))

        with jobstatus.Jobstatus(self.jobdir) as js:
            js.cleanup()

            self.assertFalse(os.path.isfile(status_path_start))


    def test_reset_can_change_status(self):
        # make a status file in job directory
        status_path_start = self.jobdir + '/STATUS_INPROGRESS'
        with open(status_path_start, 'w'): pass

        with jobstatus.Jobstatus(self.jobdir) as js:
            statusPathEnd = self.jobdir + '/STATUS_SUCCESS'

            js.reset('SUCCESS')

            self.assertFalse(os.path.isfile(status_path_start))
            self.assertTrue(os.path.isfile(statusPathEnd))


    def test_existing_status_is_untouched(self):
        # make a status file in job directory
        status_path_start = self.jobdir + '/STATUS_WARNING'
        with open(status_path_start, 'w'): pass

        with jobstatus.Jobstatus(self.jobdir) as js:
            pass

        self.assertTrue(os.path.isfile(status_path_start))


    def test_existing_warning_status_is_remembered(self):
        # make a status file in job directory
        status_path_start = self.jobdir + '/STATUS_WARNING'
        with open(status_path_start, 'w'): pass

        with jobstatus.Jobstatus(self.jobdir) as js:
            js.reset('SUCCESS')

        status_path_expected = self.jobdir + '/STATUS_SUCCESS_WITH_WARNING'

        self.assertTrue(os.path.isfile(status_path_expected))


    def test_existing_error_status_is_remembered(self):
        # make a status file in job directory
        statusPathStart = self.jobdir + '/STATUS_ERROR'
        with open(statusPathStart, 'w'): pass

        with jobstatus.Jobstatus(self.jobdir) as js:
            js.reset('SUCCESS')

        statusPathExpected = self.jobdir + '/STATUS_SUCCESS_WITH_ERROR'

        self.assertTrue(os.path.isfile(statusPathExpected))
